@@ -43,43 +43,42 @@ cand_rp_t               *cand_rp_list;
 grp_mask_t              *grp_mask_list;
 cand_rp_t               *segmented_cand_rp_list;
 grp_mask_t              *segmented_grp_mask_list;
-u_int16                 curr_bsr_fragment_tag;
-u_int8                  curr_bsr_priority;
-u_int32                 curr_bsr_address;
-u_int32                 curr_bsr_hash_mask;
-u_int16                 pim_bootstrap_timer;    /* For electing the BSR and
+uint16_t                 curr_bsr_fragment_tag;
+uint8_t                  curr_bsr_priority;
+uint32_t                 curr_bsr_address;
+uint32_t                 curr_bsr_hash_mask;
+uint16_t                 pim_bootstrap_timer;   /* For electing the BSR and
 						 * sending Cand-RP-set msgs */
-u_int8                  my_bsr_priority;
-u_int32                 my_bsr_address;
-u_int32                 my_bsr_hash_mask;
-u_int8                  cand_bsr_flag = FALSE;  /* Set to TRUE if I am
+uint8_t                  my_bsr_priority;
+uint32_t                 my_bsr_address;
+uint32_t                 my_bsr_hash_mask;
+uint8_t                  cand_bsr_flag = FALSE; /* Set to TRUE if I am
 						 * a candidate BSR */
-u_int32                 my_cand_rp_address;
-u_int8                  my_cand_rp_priority;
-u_int16                 my_cand_rp_holdtime;
-u_int16                 my_cand_rp_adv_period;  /* The locally configured
-						 * Cand-RP adv. period.
-						 */
-u_int16                 pim_cand_rp_adv_timer;
-u_int8                  cand_rp_flag  = FALSE;  /* Candidate RP flag */
+uint32_t                 my_cand_rp_address;
+uint8_t                  my_cand_rp_priority;
+uint16_t                 my_cand_rp_holdtime;
+uint16_t                 my_cand_rp_adv_period; /* The locally configured
+						 * Cand-RP adv. period. */
+uint16_t                 pim_cand_rp_adv_timer;
+uint8_t                  cand_rp_flag  = FALSE;  /* Candidate RP flag */
 struct cand_rp_adv_message_ cand_rp_adv_message;
-u_int32                 rp_my_ipv4_hashmask;
+uint32_t                 rp_my_ipv4_hashmask;
 
 
 /*
  * Local functions definition.
  */
-static cand_rp_t  *add_cand_rp          (cand_rp_t **used_cand_rp_list, u_int32 address);
+static cand_rp_t  *add_cand_rp          (cand_rp_t **used_cand_rp_list, uint32_t address);
 static grp_mask_t *add_grp_mask         (grp_mask_t **used_grp_mask_list,
-                                         u_int32 group_addr,
-                                         u_int32 group_mask,
-                                         u_int32 hash_mask);
+					 uint32_t group_addr,
+					 uint32_t group_mask,
+					 uint32_t hash_mask);
 static void       delete_grp_mask_entry (cand_rp_t **used_cand_rp_list,
-                                         grp_mask_t **used_grp_mask_list,
-                                         grp_mask_t *grp_mask_delete);
+					 grp_mask_t **used_grp_mask_list,
+					 grp_mask_t *grp_mask_delete);
 static void       delete_rp_entry       (cand_rp_t **used_cand_rp_list,
-                                         grp_mask_t **used_grp_mask_list,
-                                         cand_rp_t *cand_rp_ptr);
+					 grp_mask_t **used_grp_mask_list,
+					 cand_rp_t *cand_rp_ptr);
 
 
 void init_rp_and_bsr(void)
@@ -105,7 +104,7 @@ void init_rp_and_bsr(void)
 	curr_bsr_hash_mask = my_bsr_hash_mask;
 	SET_TIMER(pim_bootstrap_timer, bootstrap_initial_delay());
     }
-    
+
     if (cand_rp_flag != FALSE) {
 	MASKLEN_TO_MASK(RP_DEFAULT_IPV4_HASHMASKLEN, rp_my_ipv4_hashmask);
 	/* Setup the Cand-RP-Adv-Timer */
@@ -114,19 +113,19 @@ void init_rp_and_bsr(void)
 }
 
 
-u_int16 bootstrap_initial_delay(void)
+uint16_t bootstrap_initial_delay(void)
 {
-    long addr_delay;
-    long delay;
-    long log_mask;
-    int  log_of_2;
-    u_int8 best_priority;
+    uint32_t addr_delay;
+    uint32_t delay;
+    uint32_t log_mask;
+    int32_t  log_of_2;
+    uint8_t  best_priority;
 
     /*
      * The bootstrap timer initial value (if Cand-BSR).
      * It depends of the bootstrap router priority:
      * higher priority has shorter value:
-     * 
+     *
      * delay = 5 + 2 * log_2(1 + best_priority - myPriority) + addr_delay;
      *
      *    best_priority = Max(storedPriority, myPriority);
@@ -148,8 +147,8 @@ u_int16 bootstrap_initial_delay(void)
 	for (log_of_2 = (sizeof(addr_delay) << 3) - 1 ; log_of_2; log_of_2--) {
 	    if (addr_delay & log_mask)
 		break;
-	    else
-		log_mask >>= 1;  /* Start shifting `1` on right */
+
+	    log_mask >>= 1;  /* Start shifting `1` on right */
 	}
 	addr_delay = log_of_2 / 16;
     } else {
@@ -163,24 +162,24 @@ u_int16 bootstrap_initial_delay(void)
     for (log_of_2 = (sizeof(delay) << 3) - 1 ; log_of_2; log_of_2--) {
 	if (delay & log_mask)
 	    break;
-	else
-	    log_mask >>= 1;  /* Start shifting `1` on right */
+
+	log_mask >>= 1;  /* Start shifting `1` on right */
     }
 
     delay = 5 + 2 * log_of_2 + addr_delay;
 
-    return (u_int16)delay;
+    return (uint16_t)delay;
 }
-    
 
-static cand_rp_t *add_cand_rp(cand_rp_t **used_cand_rp_list, u_int32 address)
+
+static cand_rp_t *add_cand_rp(cand_rp_t **used_cand_rp_list, uint32_t address)
 {
     cand_rp_t *prev = NULL;
     cand_rp_t *next;
     cand_rp_t *ptr;
     rpentry_t *entry;
-    u_int32 addr_h = ntohl(address);
-    
+    uint32_t addr_h = ntohl(address);
+
     /* The ordering is the bigger first */
     for (next = *used_cand_rp_list; next; prev = next, next = next->next) {
 	if (ntohl(next->rpentry->address) > addr_h)
@@ -191,7 +190,7 @@ static cand_rp_t *add_cand_rp(cand_rp_t **used_cand_rp_list, u_int32 address)
 	else
 	    break;
     }
-    
+
     /* Create and insert the new entry between prev and next */
     ptr = (cand_rp_t *)calloc(1, sizeof(cand_rp_t));
     if (!ptr)
@@ -236,27 +235,27 @@ static cand_rp_t *add_cand_rp(cand_rp_t **used_cand_rp_list, u_int32 address)
 }
 
 
-static grp_mask_t *add_grp_mask(grp_mask_t **used_grp_mask_list, u_int32 group_addr, u_int32 group_mask, u_int32 hash_mask)
+static grp_mask_t *add_grp_mask(grp_mask_t **used_grp_mask_list, uint32_t group_addr, uint32_t group_mask, uint32_t hash_mask)
 {
     grp_mask_t *prev = NULL;
     grp_mask_t *next;
     grp_mask_t *ptr;
-    u_int32 prefix_h = ntohl(group_addr & group_mask);
+    uint32_t prefix_h = ntohl(group_addr & group_mask);
 
     /* The ordering of group_addr is: bigger first */
     for (next = *used_grp_mask_list; next; prev = next, next = next->next) {
-        if (ntohl(next->group_addr & next->group_mask) > prefix_h)
-            continue;
+	if (ntohl(next->group_addr & next->group_mask) > prefix_h)
+	    continue;
 
-        /* The ordering of group_mask is: bigger (longer) first */
-        if ((next->group_addr & next->group_mask) == (group_addr & group_mask)) {
-            if (ntohl(next->group_mask) > ntohl(group_mask))
-                continue;
-            else if (next->group_mask == group_mask)
-                return next;
-            else
-                break;
-        }
+	/* The ordering of group_mask is: bigger (longer) first */
+	if ((next->group_addr & next->group_mask) == (group_addr & group_mask)) {
+	    if (ntohl(next->group_mask) > ntohl(group_mask))
+		continue;
+	    else if (next->group_mask == group_mask)
+		return next;
+	    else
+		break;
+	}
     }
 
     ptr = (grp_mask_t *)calloc(1, sizeof(grp_mask_t));
@@ -289,13 +288,13 @@ static grp_mask_t *add_grp_mask(grp_mask_t **used_grp_mask_list, u_int32 group_a
  */
 rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
 				 grp_mask_t **used_grp_mask_list,
-				 u_int32 rp_addr,
-				 u_int8  rp_priority,
-				 u_int16 rp_holdtime,
-				 u_int32 group_addr,
-				 u_int32 group_mask,
-				 u_int32 bsr_hash_mask,
-				 u_int16 fragment_tag)
+				 uint32_t rp_addr,
+				 uint8_t  rp_priority,
+				 uint16_t rp_holdtime,
+				 uint32_t group_addr,
+				 uint32_t group_mask,
+				 uint32_t bsr_hash_mask,
+				 uint16_t fragment_tag)
 {
     cand_rp_t *cand_rp_ptr;
     grp_mask_t *mask_ptr;
@@ -305,8 +304,8 @@ rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
     rp_grp_entry_t *entry_prev = NULL;
     grpentry_t *grpentry_ptr_prev;
     grpentry_t *grpentry_ptr_next;
-    u_int32 rp_addr_h;
-    u_int8 old_highest_priority = ~0;  /* Smaller value means "higher" */
+    uint32_t rp_addr_h;
+    uint8_t old_highest_priority = ~0;  /* Smaller value means "higher" */
 
     /* Input data verification */
     if (!inet_valid_host(rp_addr))
@@ -331,7 +330,7 @@ rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
 		return NULL;
 	}
     }
-#endif /* 0 */    
+#endif /* 0 */
 
     cand_rp_ptr = add_cand_rp(used_cand_rp_list, rp_addr);
     if (cand_rp_ptr == NULL) {
@@ -340,7 +339,7 @@ rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
 			    group_addr, group_mask);
 	return NULL;
     }
-    
+
     rpentry_ptr = cand_rp_ptr->rpentry;
     SET_TIMER(rpentry_ptr->timer, rp_holdtime);
     rp_addr_h = ntohl(rp_addr);
@@ -360,9 +359,9 @@ rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
 	    continue;
 	if (entry_next->priority > rp_priority)
 	    break;
-	
+
 	/*
-	 * Here we don't care about higher/lower addresses, because 
+	 * Here we don't care about higher/lower addresses, because
 	 * higher address does not guarantee higher hash_value,
 	 * but anyway we do order with the higher address first,
 	 * so it will be easier to find an existing entry and update the
@@ -383,7 +382,7 @@ rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
 
 	return entry_next;
     }
-    
+
     /* Create and link the new entry */
     entry_new = (rp_grp_entry_t *)calloc(1, sizeof(rp_grp_entry_t));
     if (!entry_new)
@@ -396,7 +395,7 @@ rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
 	mask_ptr->grp_rp_next = entry_new;
     else
 	entry_prev->grp_rp_next = entry_new;
-    
+
     /*
      * The rp_grp_entry chain is not ordered, so just plug
      * the new entry at the head.
@@ -406,7 +405,7 @@ rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
 	cand_rp_ptr->rp_grp_next->rp_grp_prev = entry_new;
     entry_new->rp_grp_prev = NULL;
     cand_rp_ptr->rp_grp_next = entry_new;
-    
+
     entry_new->holdtime = rp_holdtime;
     entry_new->fragment_tag = fragment_tag;
     entry_new->priority = rp_priority;
@@ -414,8 +413,16 @@ rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
     entry_new->rp = cand_rp_ptr;
     entry_new->grplink = NULL;
 
+    /* If I am BSR candidate and rp_addr is NOT hacked SSM address, then log it */
+    if( cand_bsr_flag && rp_addr != 0x0100fea9 ) {
+	uint32_t mask;
+	MASK_TO_MASKLEN(group_mask, mask);
+	logit(LOG_INFO, 0, "New RP candidate %s for group %s/%d, priority %d",
+	      inet_fmt(rp_addr, s1, sizeof(s1)), inet_fmt(group_addr, s2, sizeof(s2)), mask, rp_priority);
+    }
+
     mask_ptr->group_rp_number++;
-    
+
     if (mask_ptr->grp_rp_next->priority == rp_priority) {
 	/* The first entries are with the best priority. */
 	/* Adding this rp_grp_entry may result in group_to_rp remapping */
@@ -430,7 +437,7 @@ rp_grp_entry_t *add_rp_grp_entry(cand_rp_t  **used_cand_rp_list,
 	    }
 	}
     }
-    
+
     return entry_new;
 }
 
@@ -451,7 +458,7 @@ void delete_rp_grp_entry(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp_ma
 	entry->rp->rp_grp_next = entry->rp_grp_next;
     if (entry->rp_grp_next)
 	entry->rp_grp_next->rp_grp_prev = entry->rp_grp_prev;
-    
+
     if (entry->grp_rp_prev)
 	entry->grp_rp_prev->grp_rp_next = entry->grp_rp_next;
     else
@@ -466,7 +473,7 @@ void delete_rp_grp_entry(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp_ma
 
     if (entry->rp->rp_grp_next == NULL)
 	delete_rp_entry(used_cand_rp_list, used_grp_mask_list, entry->rp);
-    
+
     /* Remap all affected groups */
     for (ptr = entry->grplink; ptr; ptr = ptr_next) {
 	ptr_next = ptr->rpnext;
@@ -486,7 +493,7 @@ void delete_rp_list(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_l
     rp_grp_entry_t *entry_ptr, *entry_next;
     grp_mask_t     *mask_ptr, *mask_next;
     grpentry_t     *gentry_ptr, *gentry_ptr_next;
-    
+
     for (cand_ptr = *used_cand_rp_list; cand_ptr; ) {
 	cand_next = cand_ptr->next;
 
@@ -497,7 +504,7 @@ void delete_rp_list(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_l
 	    FREE_MRTENTRY(cand_ptr->rpentry->mrtlink);
 	}
 	free(cand_ptr->rpentry);
-	
+
 	/* Free the whole chain of entry for this RP */
 	for (entry_ptr = cand_ptr->rp_grp_next; entry_ptr; entry_ptr = entry_next) {
 	    entry_next = entry_ptr->rp_grp_next;
@@ -527,28 +534,28 @@ void delete_rp_list(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_l
 }
 
 
-void delete_grp_mask(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp_mask_list, u_int32 group_addr, u_int32 group_mask)
+void delete_grp_mask(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp_mask_list, uint32_t group_addr, uint32_t group_mask)
 {
     grp_mask_t *ptr;
-    u_int32 prefix_h = ntohl(group_addr & group_mask);
+    uint32_t prefix_h = ntohl(group_addr & group_mask);
 
     for (ptr = *used_grp_mask_list; ptr; ptr = ptr->next) {
-        if (ntohl(ptr->group_addr & ptr->group_mask) > prefix_h)
-            continue;
+	if (ntohl(ptr->group_addr & ptr->group_mask) > prefix_h)
+	    continue;
 
-        if (ptr->group_addr == group_addr) {
-            if (ntohl(ptr->group_mask) > ntohl(group_mask))
-                continue;
-            else if (ptr->group_mask == group_mask)
-                break;
-            else
-                return;   /* Not found */
-        }
+	if (ptr->group_addr == group_addr) {
+	    if (ntohl(ptr->group_mask) > ntohl(group_mask))
+		continue;
+	    else if (ptr->group_mask == group_mask)
+		break;
+	    else
+		return;   /* Not found */
+	}
     }
-    
+
     if (ptr == (grp_mask_t *)NULL)
 	return;       /* Not found */
-    
+
     delete_grp_mask_entry(used_cand_rp_list, used_grp_mask_list, ptr);
 }
 
@@ -560,7 +567,7 @@ static void delete_grp_mask_entry(cand_rp_t **used_cand_rp_list, grp_mask_t **us
 
     if (grp_mask_delete == NULL)
 	return;
-    
+
     /* Remove from the grp_mask_list first */
     if (grp_mask_delete->prev)
 	grp_mask_delete->prev->next = grp_mask_delete->next;
@@ -569,7 +576,7 @@ static void delete_grp_mask_entry(cand_rp_t **used_cand_rp_list, grp_mask_t **us
 
     if (grp_mask_delete->next)
 	grp_mask_delete->next->prev = grp_mask_delete->prev;
-    
+
     /* Remove all grp_rp entries for this grp_mask */
     for (entry_ptr = grp_mask_delete->grp_rp_next; entry_ptr; entry_ptr = entry_next) {
 	entry_next = entry_ptr->grp_rp_next;
@@ -585,7 +592,7 @@ static void delete_grp_mask_entry(cand_rp_t **used_cand_rp_list, grp_mask_t **us
 	else
 	    entry_ptr->rp->rp_grp_next = entry_ptr->rp_grp_next;
 
-	if (entry_ptr->rp_grp_next != NULL) 
+	if (entry_ptr->rp_grp_next != NULL)
 	    entry_ptr->rp_grp_next->rp_grp_prev = entry_ptr->rp_grp_prev;
 
 	/* Delete the RP entry */
@@ -601,10 +608,10 @@ static void delete_grp_mask_entry(cand_rp_t **used_cand_rp_list, grp_mask_t **us
 /*
  * TODO: currently not used.
  */
-void delete_rp(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp_mask_list, u_int32 rp_addr)
+void delete_rp(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp_mask_list, uint32_t rp_addr)
 {
     cand_rp_t *ptr;
-    u_int32 rp_addr_h = ntohl(rp_addr);
+    uint32_t rp_addr_h = ntohl(rp_addr);
 
     for(ptr = *used_cand_rp_list; ptr != NULL; ptr = ptr->next) {
 	if (ntohl(ptr->rpentry->address) > rp_addr_h)
@@ -615,7 +622,7 @@ void delete_rp(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp_mask_list, u
 	else
 	    return;   /* Not found */
     }
-    
+
     if (ptr == NULL)
 	return;       /* Not found */
 
@@ -632,7 +639,7 @@ static void delete_rp_entry(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp
 
     if (cand_rp_delete == NULL)
 	return;
-    
+
     /* Remove from the cand-RP chain */
     if (cand_rp_delete->prev)
 	cand_rp_delete->prev->next = cand_rp_delete->next;
@@ -641,7 +648,7 @@ static void delete_rp_entry(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp
 
     if (cand_rp_delete->next)
 	cand_rp_delete->next->prev = cand_rp_delete->prev;
-    
+
     if (cand_rp_delete->rpentry->mrtlink) {
 	if (cand_rp_delete->rpentry->mrtlink->flags & MRTF_KERNEL_CACHE)
 	    delete_mrtentry_all_kernel_cache(cand_rp_delete->rpentry->mrtlink);
@@ -666,7 +673,7 @@ static void delete_rp_entry(cand_rp_t **used_cand_rp_list, grp_mask_t **used_grp
 
 	if (entry_ptr->grp_rp_next == NULL)
 	    delete_grp_mask_entry(used_cand_rp_list, used_grp_mask_list, entry_ptr->group);
-	
+
 	/* Remap the related groups */
 	for (grp_ptr = entry_ptr->grplink; grp_ptr; grp_ptr = grp_ptr_next) {
 	    grp_ptr_next = grp_ptr->rpnext;
@@ -692,10 +699,10 @@ int remap_grpentry(grpentry_t *grpentry_ptr)
     rp_grp_entry_t *entry_ptr;
     mrtentry_t *grp_route;
     mrtentry_t *mrtentry_ptr;
-    
+
     if (grpentry_ptr == NULL)
 	return FALSE;
-    
+
     /* Remove from the list of all groups matching to the same RP */
     if (grpentry_ptr->rpprev) {
 	grpentry_ptr->rpprev->rpnext = grpentry_ptr->rpnext;
@@ -706,7 +713,7 @@ int remap_grpentry(grpentry_t *grpentry_ptr)
 
     if (grpentry_ptr->rpnext)
 	grpentry_ptr->rpnext->rpprev = grpentry_ptr->rpprev;
-    
+
     entry_ptr = rp_grp_match(grpentry_ptr->group);
     if (entry_ptr == NULL) {
 	/* If cannot remap, delete the group */
@@ -714,7 +721,7 @@ int remap_grpentry(grpentry_t *grpentry_ptr)
 	return FALSE;
     }
     rpentry_ptr = entry_ptr->rp->rpentry;
-    
+
     /* Add to the new chain of all groups mapping to the same RP */
     grpentry_ptr->rpaddr  = rpentry_ptr->address;
     grpentry_ptr->active_rp_grp = entry_ptr;
@@ -723,7 +730,7 @@ int remap_grpentry(grpentry_t *grpentry_ptr)
 	grpentry_ptr->rpnext->rpprev = grpentry_ptr;
     grpentry_ptr->rpprev = NULL;
     entry_ptr->grplink = grpentry_ptr;
-    
+
     grp_route = grpentry_ptr->grp_route;
     if (grp_route) {
 	grp_route->upstream   = rpentry_ptr->upstream;
@@ -754,7 +761,7 @@ int remap_grpentry(grpentry_t *grpentry_ptr)
 }
 
 
-rpentry_t *rp_match(u_int32 group)
+rpentry_t *rp_match(uint32_t group)
 {
     rp_grp_entry_t *ptr;
 
@@ -765,34 +772,34 @@ rpentry_t *rp_match(u_int32 group)
     return NULL;
 }
 
-rp_grp_entry_t *rp_grp_match(u_int32 group)
+rp_grp_entry_t *rp_grp_match(uint32_t group)
 {
     grp_mask_t *mask_ptr;
     rp_grp_entry_t *entry_ptr;
     rp_grp_entry_t *best_entry = NULL;
-    u_int8 best_priority       = ~0; /* Smaller is better */
-    u_int32 best_hash_value    = 0;  /* Bigger is better */
-    u_int32 best_address_h     = 0;  /* Bigger is better */
-    u_int32 curr_hash_value    = 0;
-    u_int32 curr_address_h     = 0;
-    u_int32 group_h            = ntohl(group);
+    uint8_t best_priority       = ~0; /* Smaller is better */
+    uint32_t best_hash_value    = 0;  /* Bigger is better */
+    uint32_t best_address_h     = 0;  /* Bigger is better */
+    uint32_t curr_hash_value    = 0;
+    uint32_t curr_address_h     = 0;
+    uint32_t group_h            = ntohl(group);
 
     if (grp_mask_list == NULL)
 	return NULL;
 
     for (mask_ptr = grp_mask_list; mask_ptr; mask_ptr = mask_ptr->next) {
-	/* Search the grp_mask (group_prefix) list */
+	/* Search the grp_mask (group-prefix) list */
 	if ((group_h & ntohl(mask_ptr->group_mask))
 	    != ntohl(mask_ptr->group_mask & mask_ptr->group_addr))
 	    continue;
-	
+
 	for (entry_ptr = mask_ptr->grp_rp_next; entry_ptr; entry_ptr = entry_ptr->grp_rp_next) {
 	    if (best_priority < entry_ptr->priority)
 		break;
 
 	    curr_hash_value = RP_HASH_VALUE(group_h, mask_ptr->hash_mask, curr_address_h);
 	    curr_address_h = ntohl(entry_ptr->rp->rpentry->address);
-	    
+
 	    if (best_priority == entry_ptr->priority) {
 		/* Compare the hash_value and then the addresses */
 		if (curr_hash_value < best_hash_value)
@@ -811,15 +818,15 @@ rp_grp_entry_t *rp_grp_match(u_int32 group)
 	    best_hash_value = curr_hash_value;
 	}
     }
-    
+
     return best_entry;
 }
 
 
-rpentry_t *rp_find(u_int32 rp_address)
+rpentry_t *rp_find(uint32_t rp_address)
 {
     cand_rp_t *cand_rp_ptr;
-    u_int32 address_h = ntohl(rp_address);
+    uint32_t address_h = ntohl(rp_address);
 
     for(cand_rp_ptr = cand_rp_list; cand_rp_ptr != NULL; cand_rp_ptr = cand_rp_ptr->next) {
 	if (ntohl(cand_rp_ptr->rpentry->address) > address_h)
@@ -843,16 +850,16 @@ rpentry_t *rp_find(u_int32 rp_address)
  */
 int create_pim_bootstrap_message(char *send_buff)
 {
-    u_int8 *data_ptr;
+    uint8_t *data_ptr;
     grp_mask_t *mask_ptr;
     rp_grp_entry_t *entry_ptr;
     int datalen;
-    u_int8 masklen;
-    
+    uint8_t masklen;
+
     if (curr_bsr_address == INADDR_ANY_N)
 	return 0;
-    
-    data_ptr = (u_int8 *)(send_buff + sizeof(struct ip) + sizeof(pim_header_t));
+
+    data_ptr = (uint8_t *)(send_buff + sizeof(struct ip) + sizeof(pim_header_t));
     if (curr_bsr_address == my_bsr_address)
 	curr_bsr_fragment_tag++;
 
@@ -861,9 +868,12 @@ int create_pim_bootstrap_message(char *send_buff)
     PUT_BYTE(masklen, data_ptr);
     PUT_BYTE(curr_bsr_priority, data_ptr);
     PUT_EUADDR(curr_bsr_address, data_ptr);
-    
+
     /* TODO: XXX: No fragmentation support (yet) */
     for (mask_ptr = grp_mask_list; mask_ptr; mask_ptr = mask_ptr->next) {
+	if (IN_PIM_SSM_RANGE(mask_ptr->group_addr)) {
+	    continue;  /* Do not advertise internal virtual RP for SSM groups */
+	}
 	MASK_TO_MASKLEN(mask_ptr->group_mask, masklen);
 	PUT_EGADDR(mask_ptr->group_addr, masklen, 0, data_ptr);
 	PUT_BYTE(mask_ptr->group_rp_number, data_ptr);
@@ -877,36 +887,36 @@ int create_pim_bootstrap_message(char *send_buff)
 	    PUT_BYTE(0, data_ptr);  /* The reserved field */
 	}
     }
-    
-    datalen = (data_ptr - (u_int8 *)send_buff) - sizeof(struct ip) - sizeof(pim_header_t);
+
+    datalen = (data_ptr - (uint8_t *)send_buff) - sizeof(struct ip) - sizeof(pim_header_t);
 
     return datalen;
 }
 
 
 /*
- * Check if the rp_addr is the RP for the group corresponding to mrtentry_ptr.
+ * Check if the addr is the RP for the group corresponding to mrt.
  * Return TRUE or FALSE.
  */
-int check_mrtentry_rp(mrtentry_t *mrtentry_ptr, u_int32 rp_addr)
+int check_mrtentry_rp(mrtentry_t *mrt, uint32_t addr)
 {
     rp_grp_entry_t *ptr;
-    
-    if (mrtentry_ptr == NULL)
+
+    if (!mrt)
 	return FALSE;
 
-    if (rp_addr == INADDR_ANY_N)
+    if (addr == INADDR_ANY_N)
 	return FALSE;
 
-    ptr = mrtentry_ptr->group->active_rp_grp;
-    if (ptr == NULL)
+    ptr = mrt->group->active_rp_grp;
+    if (!ptr)
 	return FALSE;
 
-    if (mrtentry_ptr->group->rpaddr == rp_addr)
+    if (mrt->group->rpaddr == addr)
 	return TRUE;
 
     return FALSE;
-}    
+}
 
 /**
  * Local Variables:
