@@ -38,10 +38,11 @@
  * Global variables
  */
 
-/* To account for header overhead, we apx 1 byte/s = 10 bits/s (bps) */
+/* To account for header overhead, we apx 1 byte/s = 10 bits/s (bps)
+ * Note, in the new spt_threshold setting the rate is in kbps as well! */
 spt_threshold_t spt_threshold = {
     .mode     = SPT_THRESHOLD_DEFAULT_MODE,
-    .bytes    = SPT_THRESHOLD_DEFAULT_RATE * SPT_THRESHOLD_DEFAULT_INTERVAL / 10,
+    .bytes    = SPT_THRESHOLD_DEFAULT_RATE * SPT_THRESHOLD_DEFAULT_INTERVAL / 10 * 1000,
     .packets  = SPT_THRESHOLD_DEFAULT_PACKETS,
     .interval = SPT_THRESHOLD_DEFAULT_INTERVAL,
 };
@@ -360,6 +361,7 @@ void age_routes(void)
     int update_rp_iif;
     int update_src_iif;
     vifbitmap_t new_pruned_oifs;
+    int assert_timer_expired = 0;
 
     /*
      * Timing out of the global `unicast_routing_timer`
@@ -493,12 +495,23 @@ void age_routes(void)
 		    /* The (*,G) entry */
 		    /* outgoing interfaces timers */
 		    change_flag = FALSE;
+		    assert_timer_expired = 0;
+		    if (mrt_grp->flags & MRTF_ASSERTED)
+			assert_timer_expired = TIMEOUT(mrt_grp->assert_timer);
+
 		    for (vifi = 0; vifi < numvifs; vifi++) {
-			if (VIFM_ISSET(vifi, mrt_grp->joined_oifs))
+			if (VIFM_ISSET(vifi, mrt_grp->joined_oifs)) {
 			    IF_TIMEOUT(mrt_grp->vif_timers[vifi]) {
 				VIFM_CLR(vifi, mrt_grp->joined_oifs);
 				change_flag = TRUE;
 			    }
+			}
+
+			if (assert_timer_expired) {
+			    VIFM_CLR(vifi, mrt_grp->asserted_oifs);
+			    change_flag = TRUE;
+			    mrt_grp->flags &= ~MRTF_ASSERTED;
+			}
 		    }
 
 		    if ((change_flag == TRUE) || (update_rp_iif == TRUE)) {
@@ -542,13 +555,6 @@ void age_routes(void)
 			SET_TIMER(mrt_grp->jp_timer, PIM_JOIN_PRUNE_PERIOD);
 		    }
 
-		    /* Assert timer */
-		    if (mrt_grp->flags & MRTF_ASSERTED) {
-			IF_TIMEOUT(mrt_grp->assert_timer) {
-			    /* TODO: XXX: reset the upstream router now */
-			    mrt_grp->flags &= ~MRTF_ASSERTED;
-			}
-		    }
 		    /* Register-Suppression timer */
 		    /* TODO: to reduce the kernel calls, if the timer
 		     * is running, install a negative cache entry in
@@ -573,6 +579,10 @@ void age_routes(void)
 
 		    /* outgoing interfaces timers */
 		    change_flag = FALSE;
+		    assert_timer_expired = 0;
+		    if (mrt_srcs->flags & MRTF_ASSERTED)
+			assert_timer_expired = TIMEOUT(mrt_srcs->assert_timer);
+
 		    for (vifi = 0; vifi < numvifs; vifi++) {
 			if (VIFM_ISSET(vifi, mrt_srcs->joined_oifs)) {
 			    /* TODO: checking for reg_num_vif is slow! */
@@ -583,6 +593,12 @@ void age_routes(void)
 				}
 			    }
 			}
+
+			if (assert_timer_expired) {
+			    VIFM_CLR(vifi, mrt_srcs->asserted_oifs);
+			    change_flag = TRUE;
+			    mrt_srcs->flags &= ~MRTF_ASSERTED;
+			}
 		    }
 
 		    update_src_iif = FALSE;
@@ -598,18 +614,17 @@ void age_routes(void)
 				 */
 				delete_mrtentry(mrt_srcs);
 				continue;
-			    } else {
-				/* iif info found */
-				if ((srcentry_save.incoming != mrt_srcs->incoming) ||
-				    (srcentry_save.upstream != mrt_srcs->upstream)) {
-				    /* Route change has occur */
-				    update_src_iif = TRUE;
-				    mrt_srcs->incoming = mrt_srcs->source->incoming;
-				    mrt_srcs->upstream = mrt_srcs->source->upstream;
-				}
 			    }
-			}
-			else {
+
+			    /* iif info found */
+			    if ((srcentry_save.incoming != mrt_srcs->incoming) ||
+				(srcentry_save.upstream != mrt_srcs->upstream)) {
+				/* Route change has occur */
+				update_src_iif = TRUE;
+				mrt_srcs->incoming = mrt_srcs->source->incoming;
+				mrt_srcs->upstream = mrt_srcs->source->upstream;
+			    }
+			} else {
 			    /* (S,G)RPBit with iif toward RP */
 			    if ((rpentry_save.upstream != mrt_srcs->upstream) ||
 				(rpentry_save.incoming != mrt_srcs->incoming)) {
@@ -693,14 +708,6 @@ void age_routes(void)
 			SET_TIMER(mrt_srcs->jp_timer, PIM_JOIN_PRUNE_PERIOD);
 		    }
 
-		    /* Assert timer */
-		    if (mrt_srcs->flags & MRTF_ASSERTED) {
-			IF_TIMEOUT(mrt_srcs->assert_timer) {
-			    /* TODO: XXX: reset the upstream router now */
-			    mrt_srcs->flags &= ~MRTF_ASSERTED;
-			}
-		    }
-
 		    /* Register-Suppression timer */
 		    /* TODO: to reduce the kernel calls, if the timer
 		     * is running, install a negative cache entry in
