@@ -50,7 +50,7 @@
  */
 #ifndef VIFM_SET
 
-typedef	u_int32 vifbitmap_t;
+typedef	uint32_t vifbitmap_t;
 
 #define	VIFM_SET(n, m)			((m) |=  (1 << (n)))
 #define	VIFM_CLR(n, m)			((m) &= ~(1 << (n)))
@@ -79,7 +79,7 @@ typedef	u_int32 vifbitmap_t;
  * containing two variables of a native machine type.  If you
  * have a native type that's bigger than a long, define it below.
  */
-#define	NBRTYPE		u_long
+#define	NBRTYPE		uint32_t
 #define NBRBITS		sizeof(NBRTYPE) * 8
 
 typedef struct {
@@ -129,16 +129,17 @@ typedef struct {
  */
 struct uvif {
     u_int	     uv_flags;	    /* VIFF_ flags defined below            */
-    u_char	     uv_metric;     /* cost of this vif                     */
-    u_char	     uv_admetric;   /* advertised cost of this vif          */
-    u_char	     uv_threshold;  /* min ttl required to forward on vif   */
+    uint8_t	     uv_metric;     /* cost of this vif                     */
+    uint8_t	     uv_admetric;   /* advertised cost of this vif          */
+    uint8_t	     uv_threshold;  /* min ttl required to forward on vif   */
     u_int	     uv_rate_limit; /* rate limit on this vif               */
-    u_int32	     uv_lcl_addr;   /* local address of this vif            */
-    u_int32	     uv_rmt_addr;   /* remote end-point addr (tunnels only) */
-    u_int32	     uv_dst_addr;   /* destination for DVMRP/PIM messages   */
-    u_int32	     uv_subnet;     /* subnet number         (phyints only) */
-    u_int32	     uv_subnetmask; /* subnet mask           (phyints only) */
-    u_int32	     uv_subnetbcast;/* subnet broadcast addr (phyints only) */
+    int              uv_mtu;        /* Initially interface MTU, then PMTU   */
+    uint32_t	     uv_lcl_addr;   /* local address of this vif            */
+    uint32_t	     uv_rmt_addr;   /* remote end-point addr (tunnels only) */
+    uint32_t	     uv_dst_addr;   /* destination for DVMRP/PIM messages   */
+    uint32_t	     uv_subnet;     /* subnet number         (phyints only) */
+    uint32_t	     uv_subnetmask; /* subnet mask           (phyints only) */
+    uint32_t	     uv_subnetbcast;/* subnet broadcast addr (phyints only) */
     char	     uv_name[IFNAMSIZ]; /* interface name                   */
     struct listaddr *uv_groups;     /* list of local groups  (phyints only) */
     struct listaddr *uv_dvmrp_neighbors; /* list of neighboring routers     */
@@ -150,9 +151,11 @@ struct uvif {
     int		     uv_leaf_timer; /* time until this vif is considrd leaf */
     struct phaddr   *uv_addrs;	    /* Additional subnets on this vif       */
     struct vif_filter *uv_filter;   /* Route filters on this vif	    */
-    u_int16	    uv_pim_hello_timer;/* timer for sending PIM hello msgs  */
-    u_int16	    uv_gq_timer;    /* Group Query timer        	    */
-    u_int16         uv_jp_timer;    /* The Join/Prune timer                 */
+    uint16_t	    uv_hello_timer; /* Timer for sending PIM hello msgs     */
+    uint32_t        uv_dr_prio;     /* PIM Hello DR Priority                */
+    uint32_t        uv_genid;       /* Random PIM Hello Generation ID       */
+    uint16_t	    uv_gq_timer;    /* Group Query timer        	    */
+    uint16_t        uv_jp_timer;    /* The Join/Prune timer                 */
     int             uv_local_pref;  /* default local preference for assert  */
     int             uv_local_metric;/* default local metric for assert      */
     struct pim_nbr_entry *uv_pim_neighbors; /* list of PIM neighbor routers */
@@ -179,19 +182,20 @@ struct uvif {
 #define VIFF_POINT_TO_POINT     0x100000       /* point-to-point link       */
 #define VIFF_PIM_NBR            0x200000       /* PIM neighbor              */
 #define VIFF_DVMRP_NBR          0x400000       /* DVMRP neighbor            */
+#define VIFF_IGMPV2	        0x800000       /* Act as an IGMPv2 Router   */
 
 struct phaddr {
     struct phaddr   *pa_next;
-    u_int32	     pa_subnet;		/* extra subnet			*/
-    u_int32	     pa_subnetmask;	/* netmask of extra subnet	*/
-    u_int32	     pa_subnetbcast;	/* broadcast of extra subnet	*/
+    uint32_t	     pa_subnet;		/* extra subnet			*/
+    uint32_t	     pa_subnetmask;	/* netmask of extra subnet	*/
+    uint32_t	     pa_subnetbcast;	/* broadcast of extra subnet	*/
 };
 
 /* The Access Control List (list with scoped addresses) member */
 struct vif_acl {
     struct vif_acl  *acl_next;	    /* next acl member         */
-    u_int32	     acl_addr;	    /* Group address           */
-    u_int32	     acl_mask;	    /* Group addr. mask        */
+    uint32_t	     acl_addr;	    /* Group address           */
+    uint32_t	     acl_mask;	    /* Group addr. mask        */
 };
 
 struct vif_filter {
@@ -205,28 +209,30 @@ struct vif_filter {
 
 struct vf_element {
     struct vf_element  *vfe_next;
-    u_int32		vfe_addr;
-    u_int32		vfe_mask;
+    uint32_t		vfe_addr;
+    uint32_t		vfe_mask;
     int			vfe_flags;
 #define	VFEF_EXACT	0x0001
 };
 
 struct listaddr {
     struct listaddr *al_next;		/* link to next addr, MUST BE FIRST */
-    u_int32	     al_addr;		/* local group or neighbor address  */
-    u_long	     al_timer;		/* for timing out group or neighbor */
+    uint32_t	     al_addr;		/* local group or neighbor address  */
+    struct listaddr *al_sources;	/* link to sources */
+    uint32_t	     al_timer;		/* for timing out group or neighbor */
     time_t	     al_ctime;		/* entry creation time		    */
     union {
-    	u_int32	     alu_genid;		/* generation id for neighbor       */
-    	u_int32	     alu_reporter;	/* a host which reported membership */
+	uint32_t     alu_genid;		/* generation id for neighbor       */
+	uint32_t     alu_reporter;	/* a host which reported membership */
     } al_alu;
-    u_char	     al_pv;		/* router protocol version	    */
-    u_char	     al_mv;		/* router mrouted version	    */
-    u_char	     al_old;            /* time since heard old report      */
-    u_char	     al_index;		/* neighbor index		    */
-    u_long	     al_timerid;        /* timer for group membership	    */
-    u_long	     al_query;		/* timer for repeated leave query   */
-    u_int16	     al_flags;		/* flags related to this neighbor   */
+    uint8_t	     al_pv;		/* router protocol version	    */
+    uint8_t	     al_mv;		/* router mrouted version	    */
+    uint8_t	     al_old;            /* time since heard old report      */
+    uint8_t	     al_index;		/* neighbor index		    */
+    uint32_t	     al_timerid;        /* timer for group membership	    */
+    uint32_t	     al_query;		/* timer for repeated leave query   */
+    uint16_t	     al_flags;		/* flags related to this neighbor   */
+    u_long	     al_versiontimer;	/* timer for version switch    */
 };
 #define	al_genid	al_alu.alu_genid
 #define	al_reporter	al_alu.alu_reporter
@@ -257,3 +263,12 @@ struct rpfctl {
     struct in_addr rpfneighbor;/* next hop towards the source */
     vifi_t iif; /* the incoming interface to reach the next hop */
 };
+
+/**
+ * Local Variables:
+ *  version-control: t
+ *  indent-tabs-mode: t
+ *  c-file-style: "ellemtel"
+ *  c-basic-offset: 4
+ * End:
+ */
